@@ -190,8 +190,17 @@ xcf_load_image (Gimp    *gimp,
 
       /* add the layer to the image if its not the floating selection */
       if (layer != info->floating_sel)
-        gimp_image_add_layer (gimage, layer,
-                              gimp_container_num_children (gimage->layers));
+        {
+          gint position = gimp_container_num_children (gimage->layers);
+
+          /*  GIMP 2.2 wants an alpha channel for layers above the
+           *  background layer
+           */
+          if (position > 0)
+            gimp_layer_add_alpha (layer);
+
+          gimp_image_add_layer (gimage, layer, position);
+        }
 
       /* restore the saved position so we'll be ready to
        *  read the next offset.
@@ -229,7 +238,8 @@ xcf_load_image (Gimp    *gimp,
 
       /* add the channel to the image if its not the selection */
       if (channel != gimage->selection_mask)
-        gimp_image_add_channel (gimage, channel, -1);
+        gimp_image_add_channel (gimage, channel,
+                                gimp_container_num_children (gimage->channels));
 
       /* restore the saved position so we'll be ready to
        *  read the next offset.
@@ -514,8 +524,9 @@ xcf_load_image_props (XcfInfo   *info,
               {
                 if (base + prop_size != info->cp)
                   {
-                    g_warning ("Mismatch in PROP_VECTORS size: skipping %d bytes.",
-                               base + prop_size - info->cp);
+                    g_printerr ("Mismatch in PROP_VECTORS size: "
+                                "skipping %d bytes.\n",
+                                base + prop_size - info->cp);
                     xcf_seek_pos (info, base + prop_size, NULL);
                   }
               }
@@ -531,18 +542,22 @@ xcf_load_image_props (XcfInfo   *info,
 
         default:
 #ifdef GIMP_UNSTABLE
-          g_printerr ("unexpected/unknown image property: %d (skipping)",
+          g_printerr ("unexpected/unknown image property: %d (skipping)\n",
                       prop_type);
 #endif
           {
+            gsize  size = prop_size;
             guint8 buf[16];
             guint  amount;
 
-            while (prop_size > 0)
+            while (size > 0)
               {
-                amount = MIN (16, prop_size);
+                if (feof (info->fp))
+                  return FALSE;
+
+                amount = MIN (16, size);
                 info->cp += xcf_read_int8 (info->fp, buf, amount);
-                prop_size -= MIN (16, amount);
+                size -= MIN (16, amount);
               }
           }
           break;
@@ -673,19 +688,23 @@ xcf_load_layer_props (XcfInfo   *info,
           break;
 
         default:
+#ifdef GIMP_UNSTABLE
+          g_printerr ("unexpected/unknown layer property: %d (skipping)\n",
+                      prop_type);
+#endif
           {
+            gsize  size = prop_size;
             guint8 buf[16];
             guint  amount;
 
-#ifdef GIMP_UNSTABLE
-            g_printerr ("unexpected/unknown layer property: %d (skipping)",
-                        prop_type);
-#endif
-            while (prop_size > 0)
+            while (size > 0)
               {
-                amount = MIN (16, prop_size);
+                if (feof (info->fp))
+                  return FALSE;
+
+                amount = MIN (16, size);
                 info->cp += xcf_read_int8 (info->fp, buf, amount);
-                prop_size -= MIN (16, amount);
+                size -= MIN (16, amount);
               }
           }
           break;
@@ -807,19 +826,22 @@ xcf_load_channel_props (XcfInfo      *info,
 
         default:
 #ifdef GIMP_UNSTABLE
-          g_printerr ("unexpected/unknown channel property: %d (skipping)",
+          g_printerr ("unexpected/unknown channel property: %d (skipping)\n",
                       prop_type);
 #endif
-
           {
+            gsize  size = prop_size;
             guint8 buf[16];
-            guint amount;
+            guint  amount;
 
-            while (prop_size > 0)
+            while (size > 0)
               {
-                amount = MIN (16, prop_size);
+                if (feof (info->fp))
+                  return FALSE;
+
+                amount = MIN (16, size);
                 info->cp += xcf_read_int8 (info->fp, buf, amount);
-                prop_size -= MIN (16, amount);
+                size -= MIN (16, amount);
               }
           }
           break;
@@ -923,6 +945,9 @@ xcf_load_layer (XcfInfo   *info,
       layer_mask->edit_mask  = edit_mask;
       layer_mask->show_mask  = show_mask;
 
+      /*  GIMP 2.2 wants an alpha channel for layers with masks  */
+      gimp_layer_add_alpha (layer);
+
       gimp_layer_add_mask (layer, layer_mask, FALSE);
     }
 
@@ -1281,6 +1306,15 @@ xcf_load_tile_rle (XcfInfo *info,
   gint nmemb_read_successfully;
   guchar *xcfdata, *xcfodata, *xcfdatalimit;
 
+  /* Workaround for bug #357809: avoid crashing on g_malloc() and skip
+   * this tile (return TRUE without storing data) as if it did not
+   * contain any data.  It is better than returning FALSE, which would
+   * skip the whole hierarchy while there may still be some valid
+   * tiles in the file.
+   */
+  if (data_length <= 0)
+    return TRUE;
+
   data = tile_data_pointer (tile, 0, 0);
   bpp = tile_bpp (tile);
 
