@@ -17,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: pnm.c,v 1.26 2000/12/17 16:50:00 mitch Exp $ */
+/* $Id: pnm.c 15909 2004-12-12 17:18:03Z weskaggs $ */
 
 /*
  * The pnm reading and writing code was written from scratch by Erik Nygren
@@ -27,6 +27,7 @@
 
 #include "config.h"
 
+#include <errno.h>
 #include <setjmp.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -34,7 +35,6 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -97,11 +97,6 @@ typedef struct
   gint  raw;  /*  raw or ascii  */
 } PNMSaveVals;
 
-typedef struct
-{
-  gint  run;  /*  run  */
-} PNMSaveInterface;
-
 #define BUFLEN 512		/* The input buffer size for data returned
 				 * from the scanner.  Note that lines
 				 * aren't allowed to be over 256 characters
@@ -113,19 +108,17 @@ typedef struct
 /* Declare some local functions.
  */
 static void   query      (void);
-static void   run        (gchar   *name,
-                          gint     nparams,
-                          GimpParam  *param,
-                          gint    *nreturn_vals,
-                          GimpParam **return_vals);
-static gint32 load_image (gchar  *filename);
-static gint   save_image (gchar  *filename,
-			  gint32  image_ID,
-			  gint32  drawable_ID);
+static void   run        (const gchar      *name,
+                          gint              nparams,
+                          const GimpParam  *param,
+                          gint             *nreturn_vals,
+                          GimpParam       **return_vals);
+static gint32 load_image (const gchar      *filename);
+static gint   save_image (const gchar      *filename,
+			  gint32            image_ID,
+			  gint32            drawable_ID);
 
 static gint   save_dialog              (void);
-static void   save_ok_callback         (GtkWidget *widget,
-					gpointer   data);
 
 static void   pnm_load_ascii           (PNMScanner *scan,
 					PNMInfo    *info,
@@ -199,11 +192,6 @@ static PNMSaveVals psvals =
   TRUE     /* raw? or ascii */
 };
 
-static PNMSaveInterface psint =
-{
-  FALSE     /* run */
-};
-
 
 MAIN ()
 
@@ -220,9 +208,6 @@ query (void)
   {
     { GIMP_PDB_IMAGE, "image", "Output image" }
   };
-  static gint nload_args = sizeof (load_args) / sizeof (load_args[0]);
-  static gint nload_return_vals = (sizeof (load_return_vals) /
-				   sizeof (load_return_vals[0]));
 
   static GimpParamDef save_args[] =
   {
@@ -233,7 +218,6 @@ query (void)
     { GIMP_PDB_STRING,   "raw_filename", "The name of the file to save the image in" },
     { GIMP_PDB_INT32,    "raw",          "Specify non-zero for raw output, zero for ascii output" }
   };
-  static gint nsave_args = sizeof (save_args) / sizeof (save_args[0]);
 
   gimp_install_procedure ("file_pnm_load",
                           "loads files of the pnm file format",
@@ -241,47 +225,75 @@ query (void)
                           "Erik Nygren",
                           "Erik Nygren",
                           "1996",
-                          "<Load>/PNM",
+                          N_("PNM Image"),
 			  NULL,
                           GIMP_PLUGIN,
-                          nload_args, nload_return_vals,
+                          G_N_ELEMENTS (load_args),
+                          G_N_ELEMENTS (load_return_vals),
                           load_args, load_return_vals);
 
-  gimp_install_procedure ("file_pnm_save",
-                          "saves files in the pnm file format",
-                          "PNM saving handles all image types except those with alpha channels.",
-                          "Erik Nygren",
-                          "Erik Nygren",
-                          "1996",
-                          "<Save>/PNM",
-			  "RGB, GRAY, INDEXED",
-                          GIMP_PLUGIN,
-                          nsave_args, 0,
-                          save_args, NULL);
-
+  gimp_register_file_handler_mime ("file_pnm_load", "image/x-portable-anymap");
   gimp_register_magic_load_handler ("file_pnm_load",
-				    "pnm,ppm,pgm,pbm",
+                                    "pnm,ppm,pgm,pbm",
 				    "",
 				    "0,string,P1,0,string,P2,0,string,P3,0,"
 				    "string,P4,0,string,P5,0,string,P6");
-  gimp_register_save_handler       ("file_pnm_save",
-				    "pnm,ppm,pgm",
-				    "");
+
+  gimp_install_procedure ("file_pnm_save",
+                          "saves files in the pnm file format",
+                          "PNM saving handles all image types without transparency.",
+                          "Erik Nygren",
+                          "Erik Nygren",
+                          "1996",
+                          N_("PNM image"),
+			  "RGB, GRAY, INDEXED",
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (save_args), 0,
+                          save_args, NULL);
+
+  gimp_install_procedure ("file_pgm_save",
+                          "saves files in the pnm file format",
+                          "PGM saving produces grayscale images without transparency.",
+                          "Erik Nygren",
+                          "Erik Nygren",
+                          "1996",
+                          N_("PGM image"),
+			  "RGB, GRAY, INDEXED",
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (save_args), 0,
+                          save_args, NULL);
+
+  gimp_install_procedure ("file_ppm_save",
+                          "saves files in the pnm file format",
+                          "PPM saving handles RGB images without transparency.",
+                          "Erik Nygren",
+                          "Erik Nygren",
+                          "1996",
+                          N_("PPM image"),
+			  "RGB, GRAY, INDEXED",
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (save_args), 0,
+                          save_args, NULL);
+
+  gimp_register_file_handler_mime ("file_pgm_save", "image/x-portable-graymap");
+  gimp_register_file_handler_mime ("file_ppm_save", "image/x-portable-pixmap");
+  gimp_register_save_handler ("file_pgm_save", "pgm", "");
+  gimp_register_save_handler ("file_ppm_save", "ppm", "");
 }
 
 static void
-run (gchar   *name,
-     gint     nparams,
-     GimpParam  *param,
-     gint    *nreturn_vals,
-     GimpParam **return_vals)
+run (const gchar      *name,
+     gint              nparams,
+     const GimpParam  *param,
+     gint             *nreturn_vals,
+     GimpParam       **return_vals)
 {
-  static GimpParam values[2];
-  GimpRunModeType  run_mode;
-  GimpPDBStatusType   status = GIMP_PDB_SUCCESS;
-  gint32        image_ID;
-  gint32        drawable_ID;
-  GimpExportReturnType export = GIMP_EXPORT_CANCEL;
+  static GimpParam  values[2];
+  GimpRunMode       run_mode;
+  GimpPDBStatusType status = GIMP_PDB_SUCCESS;
+  gint32            image_ID;
+  gint32            drawable_ID;
+  GimpExportReturn  export = GIMP_EXPORT_CANCEL;
 
   run_mode = param[0].data.d_int32;
 
@@ -290,14 +302,7 @@ run (gchar   *name,
   values[0].type          = GIMP_PDB_STATUS;
   values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 
-  if (run_mode == GIMP_RUN_NONINTERACTIVE)
-    {
-      INIT_I18N();
-    }
-  else
-    {
-      INIT_I18N_UI();
-    }
+  INIT_I18N ();
 
   if (strcmp (name, "file_pnm_load") == 0)
     {
@@ -314,21 +319,32 @@ run (gchar   *name,
 	  status = GIMP_PDB_EXECUTION_ERROR;
 	}
     }
-  else if (strcmp (name, "file_pnm_save") == 0)
+  else if (strcmp (name, "file_pnm_save") == 0
+           || strcmp (name, "file_pgm_save") == 0
+           || strcmp (name, "file_ppm_save") == 0 )
     {
       image_ID      = param[1].data.d_int32;
       drawable_ID   = param[2].data.d_int32;
-    
-      /*  eventually export the image */ 
+
+      /*  eventually export the image */
       switch (run_mode)
 	{
 	case GIMP_RUN_INTERACTIVE:
 	case GIMP_RUN_WITH_LAST_VALS:
 	  gimp_ui_init ("pnm", FALSE);
-	  export = gimp_export_image (&image_ID, &drawable_ID, "PNM", 
-				      (GIMP_EXPORT_CAN_HANDLE_RGB |
-				       GIMP_EXPORT_CAN_HANDLE_GRAY |
-				       GIMP_EXPORT_CAN_HANDLE_INDEXED));
+          if (strcmp (name, "file_pnm_save") == 0)
+            export = gimp_export_image (&image_ID, &drawable_ID, "PNM",
+                                        (GIMP_EXPORT_CAN_HANDLE_RGB |
+                                         GIMP_EXPORT_CAN_HANDLE_GRAY |
+                                         GIMP_EXPORT_CAN_HANDLE_INDEXED));
+          else if (strcmp (name, "file_pgm_save") == 0)
+            export = gimp_export_image (&image_ID, &drawable_ID, "PGM",
+                                        (GIMP_EXPORT_CAN_HANDLE_GRAY));
+          else
+            export = gimp_export_image (&image_ID, &drawable_ID, "PPM",
+                                        (GIMP_EXPORT_CAN_HANDLE_RGB |
+                                         GIMP_EXPORT_CAN_HANDLE_INDEXED));
+
 	  if (export == GIMP_EXPORT_CANCEL)
 	    {
 	      values[0].data.d_status = GIMP_PDB_CANCEL;
@@ -343,7 +359,7 @@ run (gchar   *name,
 	{
 	case GIMP_RUN_INTERACTIVE:
 	  /*  Possibly retrieve data  */
-	  gimp_get_data ("file_pnm_save", &psvals);
+	  gimp_get_data (name, &psvals);
 
 	  /*  First acquire information with a dialog  */
 	  if (! save_dialog ())
@@ -363,7 +379,7 @@ run (gchar   *name,
 
 	case GIMP_RUN_WITH_LAST_VALS:
 	  /*  Possibly retrieve data  */
-	  gimp_get_data ("file_pnm_save", &psvals);
+	  gimp_get_data (name, &psvals);
 	  break;
 
 	default:
@@ -375,7 +391,7 @@ run (gchar   *name,
 	  if (save_image (param[3].data.d_string, image_ID, drawable_ID))
 	    {
 	      /*  Store psvals data  */
-	      gimp_set_data ("file_pnm_save", &psvals, sizeof (PNMSaveVals));
+	      gimp_set_data (name, &psvals, sizeof (PNMSaveVals));
 	    }
 	  else
 	    {
@@ -395,7 +411,7 @@ run (gchar   *name,
 }
 
 static gint32
-load_image (gchar *filename)
+load_image (const gchar *filename)
 {
   GimpPixelRgn pixel_rgn;
   gint32 volatile image_ID = -1;
@@ -408,21 +424,23 @@ load_image (gchar *filename)
   PNMScanner * volatile scan;
   int ctr;
 
-  temp = g_strdup_printf (_("Loading %s:"), filename);
-  gimp_progress_init (temp);
-  g_free (temp);
-
   /* open the file */
   fd = open (filename, O_RDONLY | _O_BINARY);
 
   if (fd == -1)
     {
-      g_message (_("PNM: Can't open file %s."), filename);
+      g_message (_("Could not open '%s' for reading: %s"),
+                 gimp_filename_to_utf8 (filename), g_strerror (errno));
       return -1;
     }
 
+  temp = g_strdup_printf (_("Opening '%s'..."),
+                          gimp_filename_to_utf8 (filename));
+  gimp_progress_init (temp);
+  g_free (temp);
+
   /* allocate the necessary structures */
-  pnminfo = (PNMInfo *) g_malloc (sizeof (PNMInfo));
+  pnminfo = g_new (PNMInfo, 1);
 
   scan = NULL;
   /* set error handling */
@@ -459,21 +477,21 @@ load_image (gchar *filename)
       }
   if (!pnminfo->loader)
     {
-      g_message (_("PNM: File not in a supported format."));
+      g_message (_("File not in a supported format."));
       longjmp(pnminfo->jmpbuf,1);
     }
 
   pnmscanner_gettoken(scan, buf, BUFLEN);
   CHECK_FOR_ERROR(pnmscanner_eof(scan), pnminfo->jmpbuf,
 		  _("PNM: Premature end of file."));
-  pnminfo->xres = isdigit(*buf)?atoi(buf):0;
+  pnminfo->xres = g_ascii_isdigit(*buf)?atoi(buf):0;
   CHECK_FOR_ERROR(pnminfo->xres<=0, pnminfo->jmpbuf,
 		  _("PNM: Invalid X resolution."));
 
   pnmscanner_gettoken(scan, buf, BUFLEN);
   CHECK_FOR_ERROR(pnmscanner_eof(scan), pnminfo->jmpbuf,
 		  _("PNM: Premature end of file."));
-  pnminfo->yres = isdigit(*buf)?atoi(buf):0;
+  pnminfo->yres = g_ascii_isdigit(*buf)?atoi(buf):0;
   CHECK_FOR_ERROR(pnminfo->yres<=0, pnminfo->jmpbuf,
 		  _("PNM: Invalid Y resolution."));
 
@@ -483,7 +501,7 @@ load_image (gchar *filename)
       CHECK_FOR_ERROR(pnmscanner_eof(scan), pnminfo->jmpbuf,
 		      _("PNM: Premature end of file."));
 
-      pnminfo->maxval = isdigit(*buf)?atoi(buf):0;
+      pnminfo->maxval = g_ascii_isdigit(*buf)?atoi(buf):0;
       CHECK_FOR_ERROR(((pnminfo->maxval<=0)
 		       || (pnminfo->maxval>255 && !pnminfo->asciibody)),
 		      pnminfo->jmpbuf,
@@ -564,14 +582,14 @@ pnm_load_ascii (PNMScanner *scan,
 		switch (info->maxval)
 		  {
 		  case 255:
-		    d[b] = isdigit(*buf)?atoi(buf):0;
+		    d[b] = g_ascii_isdigit (*buf) ? atoi (buf) : 0;
 		    break;
 		  case 1:
 		    d[b] = (*buf=='0')?0xff:0x00;
 		    break;
 		  default:
-		    d[b] = (unsigned char)(255.0*(((double)(isdigit(*buf)?atoi(buf):0))
-						  / (double)(info->maxval)));
+		    d[b] = (255.0 * (((gdouble)(g_ascii_isdigit (*buf) ? atoi (buf) : 0))
+                                     / (gdouble)(info->maxval)));
 		  }
 	      }
 
@@ -591,10 +609,10 @@ pnm_load_raw (PNMScanner *scan,
 	      PNMInfo    *info,
 	      GimpPixelRgn  *pixel_rgn)
 {
-  unsigned char *data, *d;
-  int            x, y, i;
-  int            start, end, scanlines;
-  int            fd;
+  guchar *data, *d;
+  gint    x, y, i;
+  gint    start, end, scanlines;
+  gint    fd;
 
   data = g_malloc (gimp_tile_height () * info->xres * info->np);
   fd = pnmscanner_fd(scan);
@@ -617,8 +635,11 @@ pnm_load_raw (PNMScanner *scan,
 	  if (info->maxval != 255)	/* Normalize if needed */
 	    {
 	      for (x = 0; x < info->xres * info->np; x++)
-		d[x] = (unsigned char)(255.0*(double)(d[x]) / (double)(info->maxval));
-	    }
+		{
+                  d[x] = MIN (d[x], info->maxval); /* guard against overflow */
+                  d[x] = 255.0 * (gdouble) d[x] / (gdouble) info->maxval;
+                }
+            }
 
 	  d += info->xres * info->np;
 	}
@@ -647,7 +668,7 @@ pnm_load_rawpbm (PNMScanner *scan,
   fd = pnmscanner_fd(scan);
   rowlen = (int)ceil((double)(info->xres)/8.0);
   data = g_malloc (gimp_tile_height () * info->xres);
-  buf = g_malloc(rowlen*sizeof(unsigned char));
+  buf = g_new (unsigned char, rowlen);
 
   for (y = 0; y < info->yres; )
     {
@@ -746,9 +767,9 @@ pnmsaverow_ascii_indexed (PNMRowInfo    *ri,
 }
 
 static gint
-save_image (gchar  *filename,
-	    gint32  image_ID,
-	    gint32  drawable_ID)
+save_image (const gchar *filename,
+	    gint32       image_ID,
+	    gint32       drawable_ID)
 {
   GimpPixelRgn pixel_rgn;
   GimpDrawable *drawable;
@@ -780,19 +801,21 @@ save_image (gchar  *filename,
       return FALSE;
     }
 
-  temp = g_strdup_printf (_("Saving %s:"), filename);
-  gimp_progress_init (temp);
-  g_free (temp);
-
   /* open the file */
   fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC | _O_BINARY, 0644);
 
   if (fd == -1)
     {
-      g_message ("pnm: can't open \"%s\"", filename);
+      g_message (_("Could not open '%s' for writing: %s"),
+                 gimp_filename_to_utf8 (filename), g_strerror (errno));
       return FALSE;
     }
 
+  temp = g_strdup_printf (_("Saving '%s'..."),
+                          gimp_filename_to_utf8 (filename));
+  gimp_progress_init (temp);
+  g_free (temp);
+
   xres = drawable->width;
   yres = drawable->height;
 
@@ -850,11 +873,11 @@ save_image (gchar  *filename,
 
   if (drawable_type == GIMP_INDEXED_IMAGE)
     {
-      int i;
+      gint    i;
       guchar *cmap;
-      int colors;
+      gint    colors;
 
-      cmap = gimp_image_get_cmap (image_ID, &colors);
+      cmap = gimp_image_get_colormap (image_ID, &colors);
 
       for (i = 0; i < colors; i++)
 	{
@@ -888,7 +911,7 @@ save_image (gchar  *filename,
 
   /* Write the body out */
   for (ypos = 0; ypos < yres; ypos++)
-    {      
+    {
       if ((ypos % gimp_tile_height ()) == 0)
 	{
 	  yend = ypos + gimp_tile_height ();
@@ -921,32 +944,26 @@ save_dialog (void)
 {
   GtkWidget *dlg;
   GtkWidget *frame;
+  gboolean   run;
 
   dlg = gimp_dialog_new (_("Save as PNM"), "pnm",
-			 gimp_standard_help_func, "filters/pnm.html",
-			 GTK_WIN_POS_MOUSE,
-			 FALSE, TRUE, FALSE,
+                         NULL, 0,
+			 gimp_standard_help_func, "file-pnm-save",
 
-			 _("OK"), save_ok_callback,
-			 NULL, NULL, NULL, TRUE, FALSE,
-			 _("Cancel"), gtk_widget_destroy,
-			 NULL, 1, NULL, FALSE, TRUE,
+			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
 			 NULL);
 
-  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_main_quit),
-		      NULL);
-
   /*  file save type  */
-  frame = gimp_radio_group_new2 (TRUE, _("Data Formatting"),
-				 gimp_radio_button_update,
-				 &psvals.raw, (gpointer) psvals.raw,
+  frame = gimp_int_radio_group_new (TRUE, _("Data formatting"),
+				    G_CALLBACK (gimp_radio_button_update),
+				    &psvals.raw, psvals.raw,
 
-				 _("Raw"),   (gpointer) TRUE, NULL,
-				 _("Ascii"), (gpointer) FALSE, NULL,
+				    _("Raw"),   TRUE,  NULL,
+				    _("Ascii"), FALSE, NULL,
 
-				 NULL);
+				    NULL);
   gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), frame, FALSE, TRUE, 0);
 
@@ -954,20 +971,13 @@ save_dialog (void)
 
   gtk_widget_show (dlg);
 
-  gtk_main ();
-  gdk_flush ();
+  run = (gimp_dialog_run (GIMP_DIALOG (dlg)) == GTK_RESPONSE_OK);
 
-  return psint.run;
+  gtk_widget_destroy (dlg);
+
+  return run;
 }
 
-static void
-save_ok_callback (GtkWidget *widget,
-		  gpointer   data)
-{
-  psint.run = TRUE;
-
-  gtk_widget_destroy (GTK_WIDGET (data));
-}
 
 /**************** FILE SCANNER UTILITIES **************/
 
@@ -980,9 +990,9 @@ pnmscanner_create (gint fd)
 {
   PNMScanner *s;
 
-  s = (PNMScanner *)g_malloc(sizeof(PNMScanner));
+  s = g_new (PNMScanner, 1);
   s->fd = fd;
-  s->inbuf = 0;
+  s->inbuf = NULL;
   s->eof = !read(s->fd, &(s->cur), 1);
   return(s);
 }
@@ -1004,7 +1014,7 @@ static void
 pnmscanner_createbuffer (PNMScanner *s,
 			 gint        bufsize)
 {
-  s->inbuf = g_malloc(sizeof(char)*bufsize);
+  s->inbuf = g_new (char, bufsize);
   s->inbufsize = bufsize;
   s->inbufpos = 0;
   s->inbufvalidsize = read(s->fd, s->inbuf, bufsize);
@@ -1020,8 +1030,11 @@ pnmscanner_gettoken (PNMScanner *s,
 {
   int ctr=0;
 
-  pnmscanner_eatwhitespace(s);
-  while (!(s->eof) && !isspace(s->cur) && (s->cur != '#') && (ctr<bufsize))
+  pnmscanner_eatwhitespace (s);
+  while (! s->eof                   &&
+         ! g_ascii_isspace (s->cur) &&
+         (s->cur != '#')            &&
+         (ctr < bufsize))
     {
       buf[ctr++] = s->cur;
       pnmscanner_getchar(s);
@@ -1037,7 +1050,7 @@ pnmscanner_getsmalltoken (PNMScanner *s,
 			  gchar      *buf)
 {
   pnmscanner_eatwhitespace(s);
-  if (!(s->eof) && !isspace(s->cur) && (s->cur != '#'))
+  if (!(s->eof) && !g_ascii_isspace (s->cur) && (s->cur != '#'))
     {
       *buf = s->cur;
       pnmscanner_getchar(s);
@@ -1085,7 +1098,7 @@ pnmscanner_eatwhitespace (PNMScanner *s)
 	      state = 1;  /* goto comment */
 	      pnmscanner_getchar(s);
 	    }
-	  else if (!isspace(s->cur))
+	  else if (!g_ascii_isspace (s->cur))
 	    state = -1;
 	  else
 	    pnmscanner_getchar(s);
