@@ -1,4 +1,4 @@
-/* $Id: algorithms.c,v 1.5 2000/04/14 08:59:52 neo Exp $
+/* $Id: algorithms.c 12245 2004-02-01 13:22:09Z mitch $
  * Contains routines for generating mazes, somewhat intertwined with 
  * Gimp plug-in-maze specific stuff.
  *
@@ -48,6 +48,7 @@
 #include "libgimp/stdplugins-intl.h"
 
 extern MazeValues mvals;
+extern GRand     *gr;
 
 void      mazegen(gint     pos,
 		  gchar   *maz,
@@ -62,12 +63,10 @@ void      mazegen_tileable(gint     pos,
 void      prim(gint pos,
 	       gchar *maz, 
 	       guint x, 
-	       guint y, 
-	       gint rnd);
+	       guint y);
 void      prim_tileable(gchar *maz, 
 			guint x, 
-			guint y, 
-			gint rnd);
+			guint y);
 
 #define ABSMOD(A,B) ( ((A) < 0) ? (((B) + (A)) % (B)) : ((A) % (B)) )
 
@@ -273,7 +272,7 @@ print_glist(gpointer data, gpointer user_data)
    does break, let me know, and I'll go cry in a corner for a while
    before I get up the strength to re-code it. */
 void
-prim(gint pos, gchar *maz, guint x, guint y, gint rnd)
+prim(gint pos, gchar *maz, guint x, guint y)
 {
      GSList *front_cells=NULL;
      guint current;
@@ -281,6 +280,9 @@ prim(gint pos, gchar *maz, guint x, guint y, gint rnd)
      guint progress=0, max_progress;
      char d, i;
      guint c=0;
+     gint rnd = mvals.seed;
+
+     g_rand_set_seed (gr, rnd);
 
      gimp_progress_init (_("Constructing maze using Prim's Algorithm..."));
 
@@ -322,7 +324,7 @@ prim(gint pos, gchar *maz, guint x, guint y, gint rnd)
      while(g_slist_length(front_cells) > 0) {
 
 	  /* Remove one cell at random from frontier and place it in IN. */
-	  current = rand() % g_slist_length(front_cells);
+	  current = g_rand_int_range (gr, 0, g_slist_length(front_cells));
 	  pos = GPOINTER_TO_INT(g_slist_nth(front_cells,current)->data);
 
 	  front_cells=g_slist_remove(front_cells,GINT_TO_POINTER(pos));
@@ -446,7 +448,7 @@ prim(gint pos, gchar *maz, guint x, guint y, gint rnd)
 } /* prim */
 
 void
-prim_tileable(gchar *maz, guint x, guint y, gint rnd)
+prim_tileable(gchar *maz, guint x, guint y)
 {
      GSList *front_cells=NULL;
      guint current, pos;
@@ -454,6 +456,9 @@ prim_tileable(gchar *maz, guint x, guint y, gint rnd)
      guint progress=0, max_progress;
      char d, i;
      guint c=0;
+     gint rnd = mvals.seed;
+
+     g_rand_set_seed (gr, rnd);
 
      gimp_progress_init (_("Constructing tileable maze using Prim's Algorithm..."));
 
@@ -462,8 +467,8 @@ prim_tileable(gchar *maz, guint x, guint y, gint rnd)
      max_progress=x*y/4;
 
      /* Pick someplace to start. */
-     srand(rnd);
-     pos = x * 2 * (rand() % y / 2) + 2 * (rand() % x / 2);
+
+     pos = x * 2 * g_rand_int_range (gr, 0, y/2) + 2 * g_rand_int_range(gr, 0, x/2);
 
      maz[pos]=IN;
 
@@ -475,19 +480,19 @@ prim_tileable(gchar *maz, guint x, guint y, gint rnd)
 
      maz[up]=maz[down]=maz[left]=maz[right]=FRONTIER;
 
-     front_cells=g_slist_append(front_cells,(gpointer)up);
-     front_cells=g_slist_append(front_cells,(gpointer)down);
-     front_cells=g_slist_append(front_cells,(gpointer)left);
-     front_cells=g_slist_append(front_cells,(gpointer)right);
+     front_cells=g_slist_append(front_cells,GINT_TO_POINTER(up));
+     front_cells=g_slist_append(front_cells,GINT_TO_POINTER(down));
+     front_cells=g_slist_append(front_cells,GINT_TO_POINTER(left));
+     front_cells=g_slist_append(front_cells,GINT_TO_POINTER(right));
  
      /* While frontier is not empty do the following... */
      while(g_slist_length(front_cells) > 0) {
 
 	  /* Remove one cell at random from frontier and place it in IN. */
-	  current = rand() % g_slist_length(front_cells);
-	  pos = (guint)g_slist_nth(front_cells,current)->data;
+	  current = g_rand_int_range (gr, 0, g_slist_length(front_cells));
+	  pos = GPOINTER_TO_UINT(g_slist_nth(front_cells,current)->data);
 
-	  front_cells=g_slist_remove(front_cells,(gpointer)pos);
+	  front_cells=g_slist_remove(front_cells,GUINT_TO_POINTER(pos));
 	  maz[pos]=IN;
 
 	  /* If the cell has any neighbors in OUT, remove them from
@@ -502,7 +507,7 @@ prim_tileable(gchar *maz, guint x, guint y, gint rnd)
 	  switch (maz[up]) {
 	  case OUT:
 	       maz[up]=FRONTIER;
-	       front_cells=g_slist_append(front_cells,(gpointer)up); 
+	       front_cells=g_slist_append(front_cells,GINT_TO_POINTER(up));
 	       break;
 	  case IN:
 	       d=1;
@@ -513,7 +518,7 @@ prim_tileable(gchar *maz, guint x, guint y, gint rnd)
 	  switch (maz[down]) {
 	  case OUT:
 	       maz[down]=FRONTIER;
-	       front_cells=g_slist_append(front_cells,(gpointer)down); 
+	       front_cells=g_slist_append(front_cells,GINT_TO_POINTER(down));
 	       break;
 	  case IN:
 	       d=d|2;
@@ -524,7 +529,7 @@ prim_tileable(gchar *maz, guint x, guint y, gint rnd)
 	  switch (maz[left]) {
 	  case OUT:
 	       maz[left]=FRONTIER;
-	       front_cells=g_slist_append(front_cells,(gpointer)left); 
+	       front_cells=g_slist_append(front_cells,GINT_TO_POINTER(left));
 	       break;
 	  case IN:
 	       d=d|4;
@@ -535,7 +540,7 @@ prim_tileable(gchar *maz, guint x, guint y, gint rnd)
 	  switch (maz[right]) {
 	  case OUT:
 	       maz[right]=FRONTIER;
-	       front_cells=g_slist_append(front_cells,(gpointer)right); 
+	       front_cells=g_slist_append(front_cells,GINT_TO_POINTER(right));
 	       break;
 	  case IN:
 	       d=d|8;
